%
%
%  Surface Merge Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <SurpassTab>
%        <SurpassComponent name="bpSurfaces">
%          <Item name="Merge" icon="Matlab" tooltip="Merge all the surface objects that are in the same folder in a single surface object.">
%            <Command>MatlabXT::XTSurfaceMerge(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
%
%  Description:
%   
%   Merge all the surface objects that are in the same folder in a single 
%       surface object.
%
%

function XTSurfaceMerge(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

% get the spots
vSurfaces = vImarisApplication.GetFactory.ToSurfaces(vImarisApplication.GetSurpassSelection);

if ~vImarisApplication.GetFactory.IsSurfaces(vSurfaces)  
    msgbox('Please select a surfaces object!');
    return;
end

vNumberOfSurfaces = 0;

vProgressDisplay = waitbar(0, 'Merging surfaces');
vParentGroup = vSurfaces.GetParent;

vResult = vImarisApplication.GetFactory.CreateSurfaces;

vEdges = [];
vSelection = [];
vSinglesCount = 0;

vNumberOfChildren = vParentGroup.GetNumberOfChildren;
for vChildIndex = 1:vNumberOfChildren
  vObjects = vParentGroup.GetChild(vChildIndex - 1);
  if vImarisApplication.GetFactory.IsSurfaces(vObjects)
    vSurface = vImarisApplication.GetFactory.ToSurfaces(vObjects);
    
    vSize = vSurface.GetNumberOfSurfaces;
    for vIndex = 1:vSize
      vTimeIndex = vSurface.GetTimeIndex(vIndex - 1);
      vVertices = vSurface.GetVertices(vIndex - 1);
      vTriangles = vSurface.GetTriangles(vIndex - 1);
      vNormals = vSurface.GetNormals(vIndex - 1);
      
      vResult.AddSurface(vVertices, vTriangles, vNormals, vTimeIndex);
      waitbar((vChildIndex-1+vIndex/vSize)/vNumberOfChildren, vProgressDisplay)
    end
    
    vThisEdges = vSurface.GetTrackEdges;
    vThisSelection = vSurface.GetSelectedIndices;
    vEdges = [vEdges; vThisEdges + vSinglesCount];
    vSelection = [vSelection; vThisSelection + vSinglesCount];

    vSurface.SetVisible(false);
    vNumberOfSurfaces = vNumberOfSurfaces + 1;
    vSinglesCount = vSinglesCount + vSize;
    
  end
  waitbar(vChildIndex/vNumberOfChildren)
end

% copy metadata from selected surfaces
vResult.SetTrackEdges(vEdges);
vResult.SetSelectedIndices(vSelection);
vRGBA = vSurfaces.GetColorRGBA;
vResult.SetColorRGBA(vRGBA);
vResult.SetName(sprintf('%d surfaces merged (%d total)', vNumberOfSurfaces, ...
  vSinglesCount));
vParentGroup.AddChild(vResult, -1);

close(vProgressDisplay);


